import {
  createContext,
  FC,
  ReactNode,
  useContext,
  useEffect,
  useMemo,
  useState,
} from "react";
import { defaultSettings, SettingsType } from "../lib/defaultSettings";
import { logcat } from "../lib/logcatService";
import { LowDB } from "../lib/lowDB";

// settings context
type SettingsContextProps = {
  settings: SettingsType;
  settingsLoading: boolean;
};

const SettingsContext = createContext<SettingsContextProps | undefined>(
  undefined,
);

export const SettingsProvider: FC<{ children: ReactNode }> = ({ children }) => {
  logcat.log("Initializing settings...", "INFO");

  // useStates
  const [settings, setSettings] = useState<SettingsType>(defaultSettings);
  const [settingsLoading, setSettingsLoading] = useState<boolean>(true);

  // initialize a new lowdb instance for settings outside of state
  const settingsDB = useMemo(() => {
    return new LowDB<SettingsType>("settings.json", defaultSettings);
  }, []); // empty dependency array ensures this is only created once

  // function to update settings
  const updateSettings = async (updates: Partial<SettingsType>) => {
    try {
      await settingsDB.updateData((data) => {
        Object.assign(data, updates);
      });
      const newSettings = await settingsDB.readData();
      setSettings(newSettings);
      logcat.log("Settings updated successfully", "INFO");
    } catch (error) {
      logcat.log(`Failed to update settings: ${error}`, "ERROR");
    }
  };

  // fetch user settings from local on first load every time
  useEffect(() => {
    const fetchSettings = async () => {
      try {
        await settingsDB.init();
        const storedSettings = await settingsDB.readData();
        setSettings(storedSettings);
      } catch (error) {
        logcat.log(`Failed to load settings: ${error}`, "ERROR");
      } finally {
        setSettingsLoading(false);
      }
    };

    fetchSettings();
  }, [settingsDB]);

  return (
    <SettingsContext.Provider
      value={{
        settings,
        settingsLoading,
      }}
    >
      {children}
    </SettingsContext.Provider>
  );
};

export const useSettings = () => {
  const context = useContext(SettingsContext);
  if (context === undefined) {
    throw new Error("Please use useSettings only within a SettingsProvider");
  }
  return context;
};
