import { merge } from "lodash";
import { Low } from "lowdb";
import { JSONFile } from "lowdb/node";

// define a generic interface for json structure
interface Database<T> {
  data: T;
}

type UpdateCallback<T> = (data: T) => void;

export class LowDB<T> {
  private db: Low<Database<T>>;
  private defaultData: T;

  constructor(filePath: string, defaultData: T) {
    const adapter = new JSONFile<Database<T>>(filePath);
    this.db = new Low(adapter, { data: defaultData });
    this.defaultData = defaultData;
  }

  // initialize the json file and merge with default data if needed
  async init() {
    await this.db.read();
    if (!this.db.data || !this.db.data.data) {
      // initialize with default data
      this.db.data = { data: { ...this.defaultData } };
    } else {
      // merge existing data with default data to fill in missing properties
      this.db.data.data = { ...this.defaultData, ...this.db.data.data };
    }
    await this.db.write();
  }

  async readData(): Promise<T> {
    await this.db.read();
    // ensure that the data is merged with default values every time it is read
    this.db.data!.data = merge({}, this.defaultData, this.db.data!.data);
    return this.db.data!.data;
  }

  async writeData(data: T): Promise<void> {
    this.db.data!.data = data;
    await this.db.write();
  }

  // update a specific part of the json file
  async updateData(callback: UpdateCallback<T>): Promise<void> {
    await this.db.read();
    callback(this.db.data!.data);
    await this.db.write();
  }
}
